package io.xjar.boot;

import java.util.Iterator;
import java.util.jar.Attributes;
import java.util.Set;
import java.io.Closeable;
import io.xjar.XInjector;
import java.util.zip.Checksum;
import java.util.zip.CheckedOutputStream;
import java.util.zip.CRC32;
import java.io.ByteArrayOutputStream;
import java.io.ByteArrayInputStream;
import io.xjar.XKit;
import org.apache.commons.compress.archivers.zip.ZipArchiveEntry;
import java.util.jar.Manifest;
import org.apache.commons.compress.archivers.ArchiveEntry;
import io.xjar.jar.XJarEncryptor;
import io.xjar.XUnclosedOutputStream;
import io.xjar.XUnclosedInputStream;
import org.apache.commons.compress.archivers.jar.JarArchiveOutputStream;
import org.apache.commons.compress.archivers.jar.JarArchiveInputStream;
import java.util.LinkedHashSet;
import java.io.IOException;
import io.xjar.XGo;
import java.io.OutputStream;
import java.io.InputStream;
import java.io.FileOutputStream;
import java.io.FileInputStream;
import java.io.File;
import io.xjar.key.XKey;
import java.util.HashMap;
import io.xjar.XEntryFilter;
import io.xjar.jar.XJarAllEntryFilter;
import java.util.Map;
import io.xjar.XConstants;
import io.xjar.XEncryptor;
import org.apache.commons.compress.archivers.jar.JarArchiveEntry;
import io.xjar.XEntryEncryptor;

public class XBootEncryptor extends XEntryEncryptor<JarArchiveEntry> implements XEncryptor, XConstants
{
    private final Map<String, String> map;
    private final int level;
    
    public XBootEncryptor(final XEncryptor xEncryptor) {
        this(xEncryptor, (XEntryFilter<JarArchiveEntry>)new XJarAllEntryFilter());
    }
    
    public XBootEncryptor(final XEncryptor xEncryptor, final XEntryFilter<JarArchiveEntry> filter) {
        this(xEncryptor, -1, filter);
    }
    
    public XBootEncryptor(final XEncryptor xEncryptor, final int level) {
        this(xEncryptor, level, (XEntryFilter<JarArchiveEntry>)new XJarAllEntryFilter());
    }
    
    public XBootEncryptor(final XEncryptor xEncryptor, final int level, final XEntryFilter<JarArchiveEntry> filter) {
        super(xEncryptor, (XEntryFilter)filter);
        this.map = new HashMap<String, String>();
        final String jarLauncher = "org.springframework.boot.loader.JarLauncher";
        final String warLauncher = "org.springframework.boot.loader.WarLauncher";
        final String extLauncher = "org.springframework.boot.loader.PropertiesLauncher";
        this.map.put("org.springframework.boot.loader.JarLauncher", "io.xjar.boot.XJarLauncher");
        this.map.put("org.springframework.boot.loader.WarLauncher", "io.xjar.boot.XWarLauncher");
        this.map.put("org.springframework.boot.loader.PropertiesLauncher", "io.xjar.boot.XExtLauncher");
        this.level = level;
    }
    
    public void encrypt(final XKey key, final File src, final File dest) throws IOException {
        try (FileInputStream fis = new FileInputStream(src);
             FileOutputStream fos = new FileOutputStream(dest)) {
            this.encrypt(key, fis, fos);
            XGo.make(dest, key);
        }
    }
    
    public void encrypt(final XKey key, final InputStream in, final OutputStream out) throws IOException {
        JarArchiveInputStream zis = null;
        JarArchiveOutputStream zos = null;
        final Set<String> indexes = new LinkedHashSet<String>();
        try {
            zis = new JarArchiveInputStream(in);
            zos = new JarArchiveOutputStream(out);
            zos.setLevel(this.level);
            final XUnclosedInputStream nis = new XUnclosedInputStream((InputStream)zis);
            final XUnclosedOutputStream nos = new XUnclosedOutputStream((OutputStream)zos);
            final XJarEncryptor xJarEncryptor = new XJarEncryptor(this.xEncryptor, this.level, this.filter);
            Manifest manifest = null;
            JarArchiveEntry entry;
            while ((entry = zis.getNextJarEntry()) != null) {
                if (!entry.getName().startsWith(XBootEncryptor.XJAR_SRC_DIR) && !entry.getName().endsWith("XJAR-INF/")) {
                    if (entry.getName().endsWith("XJAR-INF/INDEXES.IDX")) {
                        continue;
                    }
                    if (entry.isDirectory()) {
                        final JarArchiveEntry jarArchiveEntry = new JarArchiveEntry(entry.getName());
                        jarArchiveEntry.setTime(entry.getTime());
                        zos.putArchiveEntry((ArchiveEntry)jarArchiveEntry);
                    }
                    else if (entry.getName().equals("META-INF/MANIFEST.MF")) {
                        manifest = new Manifest((InputStream)nis);
                        final Attributes attributes = manifest.getMainAttributes();
                        final String mainClass = attributes.getValue("Main-Class");
                        if (mainClass != null) {
                            attributes.putValue("Boot-Main-Class", mainClass);
                            attributes.putValue("Main-Class", this.map.get(mainClass));
                        }
                        final JarArchiveEntry jarArchiveEntry2 = new JarArchiveEntry(entry.getName());
                        jarArchiveEntry2.setTime(entry.getTime());
                        zos.putArchiveEntry((ArchiveEntry)jarArchiveEntry2);
                        manifest.write((OutputStream)nos);
                    }
                    else if (entry.getName().startsWith("BOOT-INF/classes/")) {
                        final JarArchiveEntry jarArchiveEntry = new JarArchiveEntry(entry.getName());
                        jarArchiveEntry.setTime(entry.getTime());
                        zos.putArchiveEntry((ArchiveEntry)jarArchiveEntry);
                        final XBootJarArchiveEntry xBootJarArchiveEntry = new XBootJarArchiveEntry((ZipArchiveEntry)entry);
                        final boolean filtered = this.filtrate((Object)xBootJarArchiveEntry);
                        if (filtered) {
                            indexes.add(xBootJarArchiveEntry.getName());
                        }
                        final XEncryptor encryptor = (XEncryptor)(filtered ? this.xEncryptor : this.xNopEncryptor);
                        try (OutputStream eos = encryptor.encrypt(key, (OutputStream)nos)) {
                            XKit.transfer((InputStream)nis, eos);
                        }
                    }
                    else if (entry.getName().startsWith("BOOT-INF/lib/")) {
                        final byte[] data = XKit.read((InputStream)nis);
                        final ByteArrayInputStream lib = new ByteArrayInputStream(data);
                        final boolean need = xJarEncryptor.predicate((InputStream)lib);
                        lib.reset();
                        if (need) {
                            final ByteArrayOutputStream bos = new ByteArrayOutputStream();
                            final CheckedOutputStream cos = new CheckedOutputStream(bos, new CRC32());
                            xJarEncryptor.encrypt(key, (InputStream)lib, (OutputStream)cos);
                            final JarArchiveEntry jarArchiveEntry3 = new JarArchiveEntry(entry.getName());
                            jarArchiveEntry3.setMethod(0);
                            jarArchiveEntry3.setSize((long)bos.size());
                            jarArchiveEntry3.setTime(entry.getTime());
                            jarArchiveEntry3.setCrc(cos.getChecksum().getValue());
                            zos.putArchiveEntry((ArchiveEntry)jarArchiveEntry3);
                            final ByteArrayInputStream bis = new ByteArrayInputStream(bos.toByteArray());
                            XKit.transfer((InputStream)bis, (OutputStream)nos);
                        }
                        else {
                            final JarArchiveEntry jarArchiveEntry4 = new JarArchiveEntry(entry.getName());
                            jarArchiveEntry4.setMethod(0);
                            jarArchiveEntry4.setSize(entry.getSize());
                            jarArchiveEntry4.setTime(entry.getTime());
                            jarArchiveEntry4.setCrc(entry.getCrc());
                            zos.putArchiveEntry((ArchiveEntry)jarArchiveEntry4);
                            XKit.transfer((InputStream)lib, (OutputStream)nos);
                        }
                    }
                    else {
                        final JarArchiveEntry jarArchiveEntry = new JarArchiveEntry(entry.getName());
                        jarArchiveEntry.setTime(entry.getTime());
                        zos.putArchiveEntry((ArchiveEntry)jarArchiveEntry);
                        XKit.transfer((InputStream)nis, (OutputStream)nos);
                    }
                    zos.closeArchiveEntry();
                }
            }
            if (!indexes.isEmpty()) {
                final JarArchiveEntry xjarInfDir = new JarArchiveEntry("BOOT-INF/classes/XJAR-INF/");
                xjarInfDir.setTime(System.currentTimeMillis());
                zos.putArchiveEntry((ArchiveEntry)xjarInfDir);
                zos.closeArchiveEntry();
                final JarArchiveEntry xjarInfIdx = new JarArchiveEntry("BOOT-INF/classes/XJAR-INF/INDEXES.IDX");
                xjarInfIdx.setTime(System.currentTimeMillis());
                zos.putArchiveEntry((ArchiveEntry)xjarInfIdx);
                for (final String index : indexes) {
                    zos.write(index.getBytes());
                    zos.write(XBootEncryptor.CRLF.getBytes());
                }
                zos.closeArchiveEntry();
            }
            final String mainClass2 = (manifest != null && manifest.getMainAttributes() != null) ? manifest.getMainAttributes().getValue("Main-Class") : null;
            if (mainClass2 != null) {
                XInjector.inject(zos);
            }
            zos.finish();
        }
        finally {
            XKit.close((Closeable)zis);
            XKit.close((Closeable)zos);
        }
    }
}
